package ru.dbotthepony.mc.otm.block.entity;

import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.capability.matter.IMatterGridCell;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.BlockMatterCable;
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler;
import ru.dbotthepony.mc.otm.capability.matter.IPatternStorage;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockEntityMatterCable extends BlockEntity implements IMatterGridCell {
	public BlockEntityMatterCable(BlockPos p_155229_, BlockState p_155230_) {
		super(Registry.BlockEntities.MATTER_CABLE, p_155229_, p_155230_);
		// OverdriveThatMatters.LOGGER.debug("create cable block entity {} {} {}", this, p_155229_, p_155230_);
	}

	private MatterGrid grid;

	private LazyOptional<IMatterGridCell> resolver_grid = LazyOptional.of(() -> this);
	private boolean valid = true;

	@Override
	public void invalidateCaps() {
		super.invalidateCaps();
		valid = false;
		resolver_grid.invalidate();
	}

	@Override
	public void reviveCaps() {
		super.reviveCaps();
		valid = true;
		resolver_grid = LazyOptional.of(() -> this);
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (valid && cap == MatteryCapability.MATTER_CELL)
			return resolver_grid.cast();

		return super.getCapability(cap, side);
	}

	@Override
	public void setLevel(Level p_155231_) {
		super.setLevel(p_155231_);

		if (grid == null)
			MatterGrid.scheduleDiscoverNeighbours(this, getBlockPos(), p_155231_);
	}

	@Nullable
	@Override
	public IPatternStorage getPatternStorage() {
		return null;
	}

	@Override
	public void onNeighbourMatterCell(Level level, BlockPos pos, Direction direction, IMatterGridCell cell, BlockEntity entity) {
		BlockState new_state = getBlockState().setValue(BlockMatterCable.MAPPING_CONNECTION_PROP[direction.ordinal()], true);

		if (new_state != getBlockState())
			level.setBlock(getBlockPos(), new_state, Block.UPDATE_CLIENTS);
	}

	@Override
	public void setRemoved() {
		super.setRemoved();

		if (grid != null)
			grid.remove(this);
	}

	@Nullable
	@Override
	public MatterGrid getMatterGrid() {
		return grid;
	}

	@Nullable
	@Override
	public IMatterHandler getMatterHandler() {
		return null;
	}

	@Override
	public boolean isValidMatterCell() {
		return valid;
	}

	@Override
	public void setMatterGrid(MatterGrid grid) {
		this.grid = grid;
	}
}
