package ru.dbotthepony.mc.otm.menu;

import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraftforge.network.PacketDistributor;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMatterPanel;
import ru.dbotthepony.mc.otm.capability.matter.IMatterGridListener;
import ru.dbotthepony.mc.otm.capability.matter.MatterTask;
import ru.dbotthepony.mc.otm.capability.matter.PatternState;
import ru.dbotthepony.mc.otm.matter.MatterGrid;
import ru.dbotthepony.mc.otm.network.CancelMatterTaskPacket;
import ru.dbotthepony.mc.otm.network.MatterTaskPacket;
import ru.dbotthepony.mc.otm.network.MatteryNetworking;
import ru.dbotthepony.mc.otm.network.PatternGridPacket;

import java.util.ArrayList;
import java.util.UUID;
import java.util.function.Consumer;

public class MatterPanelMenu extends MatteryMenu implements IMatterGridListener {
	public MatterPanelMenu(int p_38852_, Inventory inventory) {
		this(p_38852_, inventory, null);
	}

	private BlockEntityMatterPanel tile;
	private MatterGrid grid;

	public MatterPanelMenu(int p_38852_, Inventory inventory, BlockEntityMatterPanel tile) {
		super(Registry.Menus.MATTER_PANEL, p_38852_, inventory, tile);
		this.tile = tile;

		if (tile != null) {
			grid = tile.getMatterGrid();
			tile.attachMenu(this);

			if (grid != null) {
				grid.attach(this);
			}
		}
	}

	public void taskUpdated(MatterTask task) {
		sendNetwork(new MatterTaskPacket(true, task));
	}

	public void taskRemoved(MatterTask task) {
		sendNetwork(new MatterTaskPacket(false, task));
	}

	// client code
	public ArrayList<PatternState> patterns = new ArrayList<>();
	public ArrayList<MatterTask> tasks = new ArrayList<>();
	public int changeset = 0;

	public void networkPatternsUpdated(PatternState[] patterns) {
		changeset++;

		for (var pattern : patterns) {
			var index_of = this.patterns.indexOf(pattern);

			if (index_of != -1) {
				this.patterns.set(index_of, pattern);
			} else {
				this.patterns.add(pattern);
			}
		}
	}

	public void networkPatternsRemoved(PatternState[] patterns) {
		changeset++;

		for (var pattern : patterns) {
			this.patterns.remove(pattern);
		}
	}

	public void networkTasksUpdated(MatterTask[] tasks) {
		changeset++;

		for (var task : tasks) {
			var index_of = this.tasks.indexOf(task);

			if (index_of != -1) {
				this.tasks.set(index_of, task);
			} else {
				this.tasks.add(task);
			}

			if (watcher_update != null)
				watcher_update.accept(task);
		}
	}

	public void networkTasksRemoved(MatterTask[] tasks) {
		changeset++;

		for (var task : tasks) {
			this.tasks.remove(task);

			if (watcher_delete != null)
				watcher_delete.accept(task);
		}
	}

	private Consumer<MatterTask> watcher_delete;
	private Consumer<MatterTask> watcher_update;

	public void networkTaskWatcher(Consumer<MatterTask> watcher_update, Consumer<MatterTask> watcher_delete) {
		this.watcher_delete = watcher_delete;
		this.watcher_update = watcher_update;
	}

	// server code
	public void requestReplication(ServerPlayer ply, PatternState state, int how_much) {
		if (tile == null)
			return;

		var grid = tile.getMatterGrid();

		if (grid == null)
			return;

		var get_pattern = grid.getPattern(state);

		if (get_pattern == null) {
			OverdriveThatMatters.LOGGER.error("Received replication request from {} of {}, but it is no longer in grid", ply, state);
			return;
		}

		tile.addTask(state, how_much);
	}

	public void receiveTaskCancel(ServerPlayer ply, UUID id) {
		if (tile == null)
			return;

		tile.removeTask(id);
	}

	public void requestTaskCancel(UUID id) {
		MatteryNetworking.CHANNEL.sendToServer(new CancelMatterTaskPacket(id));
	}

	@Override
	public void onPatternAdded(PatternState state) {
		sendNetwork(new PatternGridPacket(true, state));
	}

	@Override
	public void onPatternRemoved(PatternState state) {
		sendNetwork(new PatternGridPacket(false, state));
	}

	@Override
	public void onPatternUpdated(PatternState new_state, PatternState old_state) {
		sendNetwork(new PatternGridPacket(true, new_state));
	}

	private boolean initial_send = false;

	@Override
	public void removed(Player p_38940_) {
		super.removed(p_38940_);

		if (tile != null)
			tile.deatachMenu(this);

		if (grid != null)
			grid.detach(this);
	}

	@Override
	public void broadcastChanges() {
		super.broadcastChanges();

		if (!initial_send)
			fullPatternBroadcast();
	}

	private void sendNetwork(Object packet) {
		MatteryNetworking.CHANNEL.send(PacketDistributor.PLAYER.with(() -> (ServerPlayer) inventory.player), packet);
	}

	public void fullPatternBroadcast() {
		if (inventory.player.level.isClientSide) {
			initial_send = true;
			return;
		}

		if (tile != null) {
			var grid = tile.getMatterGrid();

			if (grid != null) {
				initial_send = true;
				sendNetwork(new PatternGridPacket(true, grid.getStoredPatterns().toArray(new PatternState[0])));
			}

			sendNetwork(new MatterTaskPacket(true, tile.getAllTasks().toArray(new MatterTask[0])));
		}
	}

	@Override
	protected int getWorkingSlotStart() {
		return 0;
	}

	@Override
	protected int getWorkingSlotEnd() {
		return 0;
	}
}
